// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: Perspective.java
// Classes: Perspective
// Original Author: ics125b spring 1996
// $Id: Perspective.java,v 1.13 1997/01/24 19:14:53 jrobbins Exp $

package uci.graphedit;

import java.awt.*;
import java.util.*;
import java.io.*;

/** Class to display graphics for NetNode's */

public class Perspective extends DiagramElement {
  
  /** when the Perspective is selected, how close should the selection
   * box be to the Perspective's bbox. Needs-More-Work: this shuold be
   * in SelectionBox. */
  private static final int BORDER_WIDTH = 10;

  /** Each Perspective keeps a collection of the NetPorts that are
   * shown in this Perspective. */
  protected Vector portList;

  /** The NetNode that is being displayed */
  NetNode parentNode;

  /** The Fig's that make up the look of this Perspective */
  FigList figList;

  /** True when we want to draw the user's attention to this
   * Perspective */
  boolean _highlight = false;

  /**  Constructs a new Perspective on the given NetNode with the
   *  given Fig's. */
  public Perspective(NetNode parent_node, FigList figs) {
    Rectangle rect;

    portList = new Vector();
    rect = figs.getBoundingBox();
    figList = figs; 
    position(rect.x, rect.y);
    parentNode = parent_node;
    parentNode.addObserver(this);
  }

  /** Replys a rectangle that bounds all the Fig's in this perspective*/
  public Rectangle getBoundingBox() { return figList.getBoundingBox(); }

  /** Move the Perspective to the given position. */
  public void position(int x,int y) {
    int dx = x - position().x;
    int dy = y - position().y;
    translate(dx, dy);
  }

  /** Move the Perspective by the given amount */
  void translate(int dx,int dy) {
    position().x += dx;
    position().y += dy;
    figList.translate(dx,dy);
  }

  /** Reply a collection of ArcPerspective's for all the NetArcs that
   * are connected to NetPorts of the NetNode being
   displayed. Needs-More-Work: this code is really slow. */
  protected Vector arcPerspectives(Editor ed) {
    Vector arcs = new Vector();
    Vector arcPerspectives = new Vector();
    Enumeration curPort =  portList.elements();
    while (curPort.hasMoreElements()) {
      NetPort p = (NetPort) curPort.nextElement();
      Enumeration curArc = p.arcList.elements();
      while (curArc.hasMoreElements()) {
	NetArc a = (NetArc) curArc.nextElement();
	arcs.addElement(a);
      }
    }
    Enumeration curDE = ed.diagramElements();
    while (curDE.hasMoreElements()) {
      DiagramElement de = (DiagramElement) curDE.nextElement();
      Object owner = de.owner();
      if (owner != null && arcs.contains(owner)) {
	arcPerspectives.addElement(de);
      }
    }
    return arcPerspectives;
  }

  /** When a Perspective is damaged, all of its arcs may need
   * redrawing. */
  public void damagedIn(Editor ed) {
    Enumeration arcPers = arcPerspectives(ed).elements();
    while (arcPers.hasMoreElements()) {
      DiagramElement de = (DiagramElement) arcPers.nextElement();
      de.damagedIn(ed);
    }
    super.damagedIn(ed);
  }

  /** When a Perspective is removed, dispose all of the NetArcs and
   * the underlying NetNode. Needs-More-Work: is this right? Doesn't
   * ActionDelete call this method also? How is deletion of a
   * Perspective handled without disposing the underlying model? */
  public void removeFrom(Editor ed) {
    Enumeration arcPers = arcPerspectives(ed).elements();
    while (arcPers.hasMoreElements()) {
      DiagramElement de = (DiagramElement) arcPers.nextElement();
      de.dispose(ed);
    }
    if (parentNode != null) parentNode.dispose();
    super.removeFrom(ed);
    parentNode = null;
  }

  /** The owner of a Perspective is an underlying NetNode */
  public Object owner() { return parentNode; }

  /**  Adds a port into the current perspective */
  public void addPort(NetPort prt, Fig rep) {
    Fig old_fig = getPortFig(prt);
    if (old_fig!=null) old_fig.owner(null);
    rep.owner((Object) prt);
    portList.addElement(prt);
  }

  /** Removes a port from the current perspective */
  public void removePort(Fig rep) {
    if (rep.owner() != null) {
      portList.removeElement(rep.owner());
      rep.owner(null);
    }
  }

  /** Returns the fig list used by this perspective.  This function is
   * essential for the perspective editor since the editor works on a
   * FigList. */
  public FigList getFigList() { return figList; }

  /** Reply true iff the point lies within any of the Fig's in the
   * perspective. */
  public boolean inside(int x, int y) {
    return (figList.pick(x, y) != null);
  }

  /** Reply true iff this Perspective intersects the given Rectangle */
  public boolean intersects(Rectangle rct) {
    return getBoundingBox().intersects(rct);
  }

  /** Reply the NetPort associated with the Fig under the mouse, or
   * null if there is none. */
  public final NetPort pickPort(Point p) { return pickPort(p.x, p.y); }

  public NetPort pickPort(int x, int y) {
    Fig ret = figList.pick(x, y);
    if (ret != null) return (NetPort) ret.owner();
    else return null;
  }

  /** Reply the Fig that displays the given NetPort. */
  public Fig getPortFig(NetPort prt) {
    Fig ret = null;
    Vector list = figList.getFigList();
    int size = list.size();
    for (int i = 0; i < size; i++) {
      Fig cur_fig = (Fig) list.elementAt(i);
      if (cur_fig.owner() == prt) {
	ret = cur_fig;
	break;
      }
    }
    return ret;
  }

  /** Draws the perspective to the given Graphics */
  public void draw(Graphics g) {
    Rectangle clip = g.getClipRect();
    Rectangle bbox = getBoundingBox();
    if (clip == null || bbox.intersects(clip)) {
      figList.draw(g);
    }
    if (_highlight) {
      g.setColor(Globals.prefs().highlightColor()); /* needs-more-work */
      g.drawRect(bbox.x-3, bbox.y-3, bbox.width+6, bbox.height+6);
      g.drawRect(bbox.x-4, bbox.y-4, bbox.width+8, bbox.height+8);
    }
  }

  /** Needs-More-Work: remove this, its logic has been moved to
   * SelectionBox. */
  public void drawSelected(Graphics g) { }


  /** When a Perspective is selected the Editor should record that
   * fact in an instance of SelectionBox. */
  public Selection selectionObject() { return new SelectionBox(this); }

  /** If I get a notification that I shuold be Highlighted or
   * Unhighlight, set a flag and mark my area as damaged. */
  public void update(Observable o, Object arg) {
    if (arg.equals("Highlight")) { 
      startTrans();
      _highlight = true; 
      endTrans();
    }
    else if (arg.equals("Unhighlight")) { 
      startTrans();
      _highlight = false; 
      endTrans();
    }
  }


} /* end class Perspective */

